<?php

namespace Drupal\migrate_generator_export\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Migrate generator settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['migrate_generator_export.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'migrate_generator_export_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('migrate_generator_export.settings');

    $form['delimiter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Delimiter'),
      '#description' => $this->t('Add your delimiter (e.g., comma, semi-colon)'),
      '#maxlength' => 2,
      '#size' => 4,
      '#default_value' => ';',
      '#default_value' => $config->get('delimiter'),
    ];

    $form['enclosure'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enclosure'),
      '#description' => $this->t('Enclosure for source CSV files'),
      '#maxlength' => 2,
      '#size' => 4,
      '#default_value' => '"',
      '#default_value' => $config->get('enclosure'),
    ];
    $form['values_delimiter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Delimiter for multi-valued fields'),
      '#description' => $this->t('Delimiter for multi-valued fields.'),
      '#maxlength' => 2,
      '#size' => 4,
      '#default_value' => '|',
      '#default_value' => $config->get('values_delimiter'),
    ];
    $form['date_format'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Date/time format'),
      '#description' => $this->t('See <a href="https://www.php.net/manual/datetime.format.php#refsect1-datetime.format-parameters" target="_blank">the documentation for PHP date formats</a>.'),
      '#default_value' => $config->get('date_format'),
    ];
    $form['file_format'] = [
      '#type' => 'radios',
      '#title' => $this->t('File format'),
      '#options' => [
        'url' => $this
          ->t('Absolute URL'),
        'filepath' => $this
          ->t('Absolute file path'),
      ],
      '#default_value' => $config->get('file_format'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('migrate_generator_export.settings')
      ->set('delimiter', $form_state->getValue('delimiter'))
      ->set('enclosure', $form_state->getValue('enclosure'))
      ->set('values_delimiter', $form_state->getValue('values_delimiter'))
      ->set('date_format', $form_state->getValue('date_format'))
      ->set('file_format', $form_state->getValue('file_format'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
